#!/bin/bash
# Author:	Lazzarin Alberto
# Date:		20-06-2012
# Version	1.3
#
# This plugin check the HW status of IBM Storwize v7000.
# To use this script you need to create a linux user with his ssh certificate.
# Create the same user on the v7000, member of monitor group and upload the public ssh key.
# Try to log from linux machine to the v7000 without password, if it function you can use the plugin.
# I use the 'nagios' user to check the remote system.
# The help is included into the script.
#
#
#
# CHANGELOG
#
# 1.3 Made by Ivan Bergantin [ivan . bergantin @ gmail . com] suggested by Leandro Freitas [leandro @ nodeps . com . br]
# Add short output in "Service Status Details For Host" view, and detailed output in "Service Information"view
#
# 1.2 Made by Feilong
# Add check of mirror status between two volumes on two IBM V7000.
# It check the number of mirrors, the numbers of consitent and synchronized mirrors. If they are differents, the status returned is critical.
#
# 1.1
# Change login method from from 'plink' to ssh.
# Add "OK" and "ATTENTION" in the output.
#
# 1.0
# First release.
#

ssh=/usr/bin/ssh
exitCode=0

while getopts 'M:U:Q:d:hK:' OPT; do
  case $OPT in
    M)  storage=$OPTARG;;
    U)  user=$OPTARG;;
    K)  keyfile="-i $OPTARG";;
    Q)  query=$OPTARG;;
    h)  hlp="yes";;
    *)  unknown="yes";;
  esac
done

# usage
HELP="
    Check IBM Storwize v7000 throght ssh (GPL licence)

    usage: $0 [ -M value -U value -Q command -h [-K value] ]

    syntax:

            -M --> IP Address
            -U --> user
            -K --> ssh-key-file
            -Q --> query to storage
		lsarray
		lsdrive
		lsvdisk
		lsenclosure
		lsenclosurebattery
		lsenclosurecanister
		lsenclosurepsu
		lsenclosureslot
		lsrcrelationship
            -h --> Print This Help Screen

    Note :
    This check use ssh protocoll.
"

if [ "$hlp" = "yes" -o $# -lt 1 ]; then
        echo "$HELP"
        exit 0
fi

tmp_file=/tmp/v7000_$storage_${query}_0_$$.tmp
tmp_fil1=/tmp/v7000_$storage_${query}_1_$$.tmp
tmp_fil2=/tmp/v7000_$storage_${query}_2_$$.tmp
outputMess=""

trap 'rm -f $tmp_file $tmp_fil1; exit 1' 1 2 15
trap 'rm -f $tmp_file $tmp_fil1' 0

#echo -ne "IBM Storwize v7000 Health Check\n"

$ssh $keyfile $user@$storage $query >$tmp_fil1 2>$tmp_fil2
if [ -s $tmp_fil2 ]
then
	echo "UNKNOWN - Error executing ssh command"
	echo "$ssh $keyfile $user@$storage $query"
	cat $tmp_fil2
	exit 3
fi
if [ ! -s $tmp_fil1 ]
then
	echo "UNKNOWN - Error executing ssh command, no output"
	echo "$ssh $keyfile $user@$storage $query"
	exit 3
fi

case $query in 
	lsarray)
		cat $tmp_fil1 |sed '1d' > $tmp_file

		cat $tmp_file |awk '{printf $3}' |grep -i offline
		if [ "$?" -eq "0" ]; then
                        outputMess="$outputMess CRITICAL: MDisk OFFLINE \n"
		else
                        outputMess="$outputMess OK: MDisks \n"
		fi

		while read line
			do
				mdisk_name=$(echo "${line}" | awk '{printf $2}')
				mdisk_status=$(echo "${line}" | awk '{printf $3}')

               			if [ $mdisk_status = "online" ]; then
                       			outputMess="$outputMess OK: MDisks $mdisk_name status: $mdisk_status \n"
               			else
					outputMess="$outputMess ATTENTION: MDisks $mdisk_name status: $mdisk_status \n"
              				exitCode=2
				fi

       		done < $tmp_file
	;;

	lsdrive)
                cat $tmp_fil1 |sed '1d' > $tmp_file

                cat $tmp_file |awk '{printf $2}' |grep -i offline
                if [ "$?" -eq "0" ]; then
                        outputMess="$outputMess CRITICAL: Disk OFFLINE \n"
                else
                        outputMess="$outputMess OK: Drive \n"
                fi

		drive_total=$(/bin/cat $tmp_file |/usr/bin/wc -l)
                while read line
                        do
                                drive_n=$(echo "${line}" | awk '{printf $1}')
                                drive_status=$(echo "${line}" | awk '{printf $2}')
                                drive_role=$(echo "${line}" | awk '{printf $4}')
                                drive_type=$(echo "${line}" | awk '{printf $5}')
                                drive_capacity=$(echo "${line}" | awk '{printf $6}')
                                drive_enclosure=$(echo "${line}" | awk '{printf $10}')
                                drive_slot=$(echo "${line}" | awk '{printf $11}')

                                if [ $drive_status = "online" ]; then
                                        outputMess="$outputMess OK: Drive $drive_n is online \n"
                                else
                                        outputMess="$outputMess ATTENTION: Disk $drive_n \nstatus: $disk_status \nrole: $drive_role \ntype: $drive_type \ncapacity: $drive_capacity \nenclosure: $drive_enclosure \nslot: $drive_slot "
                                        exitCode=2
                                fi

                done < $tmp_file

	;;

	lsvdisk)
                cat $tmp_fil1 |sed '1d' > $tmp_file

                cat $tmp_file |awk '{printf $5}' |grep -i offline
                if [ "$?" -eq "0" ]; then
                        outputMess="$outputMess CRITICAL: VDisk OFFLINE \n"
                else
                        outputMess="$outputMess OK: VDisk \n"
                fi

                while read line
                        do
                                vdisk_name=$(echo "${line}" | awk '{printf $2}')
                                vdisk_status=$(echo "${line}" | awk '{printf $5}')

                                if [ $vdisk_status = "online" ]; then
                                        outputMess="$outputMess OK: VDisks $vdisk_name status: $vdisk_status \n"
                                else
                                        outputMess="$outputMess ATTENTION: VDisks $mdisk_name status: $vdisk_status \n"
                                        exitCode=2
                                fi

                done < $tmp_file
	;;

	lsenclosure)
                cat $tmp_fil1 |sed '1d' > $tmp_file

                cat $tmp_file |awk '{printf $2}' |grep -i offline
                if [ "$?" -eq "0" ]; then
                        outputMess="$outputMess CRITICAL: Enclosure OFFLINE \n"
                else
                        outputMess="$outputMess OK: Enclosure \n"
                fi

                while read line
                        do
                                enc_n=$(echo "${line}" | awk '{printf $1}')
                                enc_status=$(echo "${line}" | awk '{printf $2}')
                                enc_pn=$(echo "${line}" | awk '{printf $7}')
                                enc_sn=$(echo "${line}" | awk '{printf $8}')


                                if [ $enc_status = "online" ]; then
                                        outputMess="$outputMess OK: Enclosure $enc_n status: $enc_status \n"
                                else
                                        outputMess="$outputMess ATTENTION: Enclosure $enc_n status: $enc_status sn: $enc_sn pn: $enc_pn \n"
                                        exitCode=2
                                fi

                done < $tmp_file
	;;

	lsenclosurebattery)
                cat $tmp_fil1 |sed '1d' > $tmp_file

                cat $tmp_file |awk '{printf $3}' |grep -i offline
                if [ "$?" -eq "0" ]; then
                        outputMess="$outputMess CRITICAL: Battery OFFLINE \n"
                else
                        outputMess="$outputMess OK: Battery \n"
                fi

                while read line
                        do
                                batt_n=$(echo "${line}" | awk '{printf $2}')
                                batt_status=$(echo "${line}" | awk '{printf $3}')
                                batt_charge=$(echo "${line}" | awk '{printf $4}')
                                batt_rec=$(echo "${line}" | awk '{printf $5}')
                                batt_charge=$(echo "${line}" | awk '{printf $6}')
				batt_eol=$(echo "${line}" | awk '{printf $7}')


                                if [ $batt_status = "online" -a  $batt_rec = "no" -a $batt_charge = "100" -a $batt_eol = "no" ]; then
                                        outputMess="$outputMess OK: Battery $batt_n status: $batt_status \n"
                                else
                                        outputMess="$outputMess ATTENTION: Battery $batt_n status: $batt_statusn recharge: $batt_rec charged: $batt_charge eol: $batt_eol \n"
                                        exitCode=2
                                fi

                done < $tmp_file
	;;

	lsenclosurecanister)
                cat $tmp_fil1 |sed '1d' > $tmp_file

                cat $tmp_file |awk '{printf $3}' |grep -i offline
                if [ "$?" -eq "0" ]; then
                        outputMess="$outputMess CRITICAL: Canister OFFLINE \n"
                else
                        outputMess="$outputMess OK: Canister \n"
                fi

                while read line
                        do
                                can_id=$(echo "${line}" | awk '{printf $2}')
                                can_enc_id=$(echo "${line}" | awk '{printf $1}')
                                can_stat=$(echo "${line}" | awk '{printf $3}')
                                can_type=$(echo "${line}" | awk '{printf $4}')

                                if [ $can_stat = "online" ]; then
                                        outputMess="$outputMess OK: Canister $can_id enclosure: $can_enc_id status: $can_stat \n"
                                else
                                        outputMess="$outputMess ATTENTION: Canister $can_id enclosure: $can_enc_id status: $can_stat type: $can_type \n"
                                        exitCode=2
                                fi

                done < $tmp_file
	;;

	lsenclosurepsu)
                cat $tmp_fil1 |sed '1d' > $tmp_file

                cat $tmp_file |awk '{printf $3}' |grep -i offline
                if [ "$?" -eq "0" ]; then
                        outputMess="$outputMess CRITICAL: PSU OFFLINE \n"
                else
                        outputMess="$outputMess OK: PSU \n"
                fi

                while read line
                        do
                                psu_id=$(echo "${line}" | awk '{printf $2}')
                                psu_enc_id=$(echo "${line}" | awk '{printf $1}')
                                psu_stat=$(echo "${line}" | awk '{printf $3}')

                                if [ $psu_stat = "online" ]; then
                                        outputMess="$outputMess OK: PSU $psu_id enclosure: $psu_enc_id status: $psu_stat \n"
                                else
                                        outputMess="$outputMess ATTENTION: PSU $psu_id enclosure: $psu_enc_id status: $psu_stat \n"
                                        exitCode=2
                                fi

                done < $tmp_file
	;;

	lsenclosureslot)
                cat $tmp_fil1 |sed '1d' > $tmp_file

                cat $tmp_file |awk '{printf $3, $4}' |grep -i offline
                if [ "$?" -eq "0" ]; then
                        outputMess="$outputMess CRITICAL: EnclosureSlot OFFLINE \n"
                else
                        outputMess="$outputMess OK: EnclosureSlot \n"
                fi

                while read line
                        do
                                slt_enc_id=$(echo "${line}" | awk '{printf $1}')
                                slt_id=$(echo "${line}" | awk '{printf $2}')
                                slt_prt1_stat=$(echo "${line}" | awk '{printf $3}')
                                slt_prt2_stat=$(echo "${line}" | awk '{printf $4}')
                                slt_drv=$(echo "${line}" | awk '{printf $5}')
                                drv_id=$(echo "${line}" | awk '{printf $6}')

                                if [ $slt_prt1_stat = "online" -a $slt_prt2_stat = "online" -a $slt_drv = "yes" ]; then
                                        outputMess="$outputMess OK: Drive-$drv_id enclosure-$slt_enc_id slot-$slt_id port1-$slt_prt1_stat port2-$slt_prt2_stat\n"
                                else
                                        outputMess="$outputMess ATTENTION: Drive-$drv_id enclosure-$slt_enc_id slot-$slt_id port1-$slt_prt1_stat port2-$slt_prt2_stat \n"
                                        exitCode=2
                                fi

                done < $tmp_file
	;;

	lsrcrelationship)
                volume_mirror_prod=$(cat $tmp_fil1 | grep -c "rcrel*")
                volume_mirror_sync=$(cat $tmp_fil1 | grep -c "consistent_synchronized")
                volume_mirror_inco=$(cat $tmp_fil1 | grep -c "inconsistent_copying")
                volume_mirror_stop=$(cat $tmp_fil1 | grep -c "_stopped")

                if [ $volume_mirror_prod = $volume_mirror_sync ]; then
                    outputMess="$outputMess OK: $volume_mirror_prod mirors are consistent and synchronized \n"
		elif [ $volume_mirror_stop -gt 0 ]; then
                    outputMess="$outputMess CRITICAL: Of $volume_mirror_prod volumes, only $volume_mirror_sync are consistent and synchronized, some are stopped\n"
                    exitCode=2
		else
                    outputMess="$outputMess WARNING: Of $volume_mirror_prod volumes, only $volume_mirror_sync are consistent and synchronized, some are syncronizing\n"
                    exitCode=1
                fi
                outputMess="$outputMess|sync=$volume_mirror_sync;1;1;0;$volume_mirror_prod inconsistent=$volume_mirror_inco;1;1;0;$volume_mirror_prod stopped=$volume_mirror_stop;1;1;0;$volume_mirror_prod\n"
		tail -n +2 $tmp_fil1 | grep -v "consistent_synchronized"
	;;

	*)
		echo -ne "Command not found. \n"
		exit 3
	;;
esac

echo -ne "$outputMess"
exit $exitCode
